"""
Unit and regression test for the TADA_T2 package.
"""

# Import package, test suite, and other packages as needed
import sys

import pytest

import TADA_T2


def test_TADA_T2_imported():
    """Sample test, will always pass so long as import statement worked."""
    assert "TADA_T2" in sys.modules
